# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawCustomAttributesClient
from ...requests.booking_custom_attribute_delete_request import BookingCustomAttributeDeleteRequestParams
from ...core.request_options import RequestOptions
from ...types.bulk_delete_booking_custom_attributes_response import BulkDeleteBookingCustomAttributesResponse
from ...requests.booking_custom_attribute_upsert_request import BookingCustomAttributeUpsertRequestParams
from ...types.bulk_upsert_booking_custom_attributes_response import BulkUpsertBookingCustomAttributesResponse
from ...core.pagination import SyncPager
from ...types.custom_attribute import CustomAttribute
from ...core.jsonable_encoder import jsonable_encoder
from ...types.list_booking_custom_attributes_response import ListBookingCustomAttributesResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.retrieve_booking_custom_attribute_response import RetrieveBookingCustomAttributeResponse
from ...requests.custom_attribute import CustomAttributeParams
from ...types.upsert_booking_custom_attribute_response import UpsertBookingCustomAttributeResponse
from ...types.delete_booking_custom_attribute_response import DeleteBookingCustomAttributeResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawCustomAttributesClient
from ...core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomAttributesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomAttributesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomAttributesClient
        """
        return self._raw_client

    def batch_delete(
        self,
        *,
        values: typing.Dict[str, BookingCustomAttributeDeleteRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkDeleteBookingCustomAttributesResponse:
        """
        Bulk deletes bookings custom attributes.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        values : typing.Dict[str, BookingCustomAttributeDeleteRequestParams]
            A map containing 1 to 25 individual Delete requests. For each request, provide an
            arbitrary ID that is unique for this `BulkDeleteBookingCustomAttributes` request and the
            information needed to delete a custom attribute.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkDeleteBookingCustomAttributesResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.custom_attributes.batch_delete(
            values={"key": {"booking_id": "booking_id", "key": "key"}},
        )
        """
        response = self._raw_client.batch_delete(values=values, request_options=request_options)
        return response.data

    def batch_upsert(
        self,
        *,
        values: typing.Dict[str, BookingCustomAttributeUpsertRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkUpsertBookingCustomAttributesResponse:
        """
        Bulk upserts bookings custom attributes.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        values : typing.Dict[str, BookingCustomAttributeUpsertRequestParams]
            A map containing 1 to 25 individual upsert requests. For each request, provide an
            arbitrary ID that is unique for this `BulkUpsertBookingCustomAttributes` request and the
            information needed to create or update a custom attribute.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkUpsertBookingCustomAttributesResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.custom_attributes.batch_upsert(
            values={"key": {"booking_id": "booking_id", "custom_attribute": {}}},
        )
        """
        response = self._raw_client.batch_upsert(values=values, request_options=request_options)
        return response.data

    def list(
        self,
        booking_id: str,
        *,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        with_definitions: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[CustomAttribute]:
        """
        Lists a booking's custom attributes.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        booking_id : str
            The ID of the target [booking](entity:Booking).

        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response. This limit is advisory.
            The response might contain more or fewer results. The minimum value is 1 and the maximum value is 100.
            The default value is 20. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request. For more
            information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        with_definitions : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of each
            custom attribute. Set this parameter to `true` to get the name and description of each custom
            attribute, information about the data type, or other definition details. The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[CustomAttribute]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.bookings.custom_attributes.list(
            booking_id="booking_id",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            f"v2/bookings/{jsonable_encoder(booking_id)}/custom-attributes",
            method="GET",
            params={
                "limit": limit,
                "cursor": cursor,
                "with_definitions": with_definitions,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListBookingCustomAttributesResponse,
                    construct_type(
                        type_=ListBookingCustomAttributesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    booking_id,
                    limit=limit,
                    cursor=_parsed_next,
                    with_definitions=with_definitions,
                    request_options=request_options,
                )
                _items = _parsed_response.custom_attributes
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        booking_id: str,
        key: str,
        *,
        with_definition: typing.Optional[bool] = None,
        version: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RetrieveBookingCustomAttributeResponse:
        """
        Retrieves a bookings custom attribute.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        booking_id : str
            The ID of the target [booking](entity:Booking).

        key : str
            The key of the custom attribute to retrieve. This key must match the `key` of a custom
            attribute definition in the Square seller account. If the requesting application is not the
            definition owner, you must use the qualified key.

        with_definition : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of
            the custom attribute. Set this parameter to `true` to get the name and description of the custom
            attribute, information about the data type, or other definition details. The default value is `false`.

        version : typing.Optional[int]
            The current version of the custom attribute, which is used for strongly consistent reads to
            guarantee that you receive the most up-to-date data. When included in the request, Square
            returns the specified version or a higher version if one exists. If the specified version is
            higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveBookingCustomAttributeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.custom_attributes.get(
            booking_id="booking_id",
            key="key",
        )
        """
        response = self._raw_client.get(
            booking_id, key, with_definition=with_definition, version=version, request_options=request_options
        )
        return response.data

    def upsert(
        self,
        booking_id: str,
        key: str,
        *,
        custom_attribute: CustomAttributeParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpsertBookingCustomAttributeResponse:
        """
        Upserts a bookings custom attribute.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking_id : str
            The ID of the target [booking](entity:Booking).

        key : str
            The key of the custom attribute to create or update. This key must match the `key` of a
            custom attribute definition in the Square seller account. If the requesting application is not
            the definition owner, you must use the qualified key.

        custom_attribute : CustomAttributeParams
            The custom attribute to create or update, with the following fields:

            - `value`. This value must conform to the `schema` specified by the definition.
            For more information, see [Value data types](https://developer.squareup.com/docs/booking-custom-attributes-api/custom-attributes#value-data-types).

            - `version`. To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control for an update operation, include this optional field and specify the current version
            of the custom attribute.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpsertBookingCustomAttributeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.custom_attributes.upsert(
            booking_id="booking_id",
            key="key",
            custom_attribute={},
        )
        """
        response = self._raw_client.upsert(
            booking_id,
            key,
            custom_attribute=custom_attribute,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    def delete(
        self, booking_id: str, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteBookingCustomAttributeResponse:
        """
        Deletes a bookings custom attribute.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking_id : str
            The ID of the target [booking](entity:Booking).

        key : str
            The key of the custom attribute to delete. This key must match the `key` of a custom
            attribute definition in the Square seller account. If the requesting application is not the
            definition owner, you must use the qualified key.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteBookingCustomAttributeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.custom_attributes.delete(
            booking_id="booking_id",
            key="key",
        )
        """
        response = self._raw_client.delete(booking_id, key, request_options=request_options)
        return response.data


class AsyncCustomAttributesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomAttributesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomAttributesClient
        """
        return self._raw_client

    async def batch_delete(
        self,
        *,
        values: typing.Dict[str, BookingCustomAttributeDeleteRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkDeleteBookingCustomAttributesResponse:
        """
        Bulk deletes bookings custom attributes.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        values : typing.Dict[str, BookingCustomAttributeDeleteRequestParams]
            A map containing 1 to 25 individual Delete requests. For each request, provide an
            arbitrary ID that is unique for this `BulkDeleteBookingCustomAttributes` request and the
            information needed to delete a custom attribute.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkDeleteBookingCustomAttributesResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.custom_attributes.batch_delete(
                values={"key": {"booking_id": "booking_id", "key": "key"}},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.batch_delete(values=values, request_options=request_options)
        return response.data

    async def batch_upsert(
        self,
        *,
        values: typing.Dict[str, BookingCustomAttributeUpsertRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkUpsertBookingCustomAttributesResponse:
        """
        Bulk upserts bookings custom attributes.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        values : typing.Dict[str, BookingCustomAttributeUpsertRequestParams]
            A map containing 1 to 25 individual upsert requests. For each request, provide an
            arbitrary ID that is unique for this `BulkUpsertBookingCustomAttributes` request and the
            information needed to create or update a custom attribute.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkUpsertBookingCustomAttributesResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.custom_attributes.batch_upsert(
                values={"key": {"booking_id": "booking_id", "custom_attribute": {}}},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.batch_upsert(values=values, request_options=request_options)
        return response.data

    async def list(
        self,
        booking_id: str,
        *,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        with_definitions: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[CustomAttribute]:
        """
        Lists a booking's custom attributes.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        booking_id : str
            The ID of the target [booking](entity:Booking).

        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response. This limit is advisory.
            The response might contain more or fewer results. The minimum value is 1 and the maximum value is 100.
            The default value is 20. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request. For more
            information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        with_definitions : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of each
            custom attribute. Set this parameter to `true` to get the name and description of each custom
            attribute, information about the data type, or other definition details. The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[CustomAttribute]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.bookings.custom_attributes.list(
                booking_id="booking_id",
            )
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            f"v2/bookings/{jsonable_encoder(booking_id)}/custom-attributes",
            method="GET",
            params={
                "limit": limit,
                "cursor": cursor,
                "with_definitions": with_definitions,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListBookingCustomAttributesResponse,
                    construct_type(
                        type_=ListBookingCustomAttributesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    booking_id,
                    limit=limit,
                    cursor=_parsed_next,
                    with_definitions=with_definitions,
                    request_options=request_options,
                )
                _items = _parsed_response.custom_attributes
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        booking_id: str,
        key: str,
        *,
        with_definition: typing.Optional[bool] = None,
        version: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RetrieveBookingCustomAttributeResponse:
        """
        Retrieves a bookings custom attribute.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        booking_id : str
            The ID of the target [booking](entity:Booking).

        key : str
            The key of the custom attribute to retrieve. This key must match the `key` of a custom
            attribute definition in the Square seller account. If the requesting application is not the
            definition owner, you must use the qualified key.

        with_definition : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of
            the custom attribute. Set this parameter to `true` to get the name and description of the custom
            attribute, information about the data type, or other definition details. The default value is `false`.

        version : typing.Optional[int]
            The current version of the custom attribute, which is used for strongly consistent reads to
            guarantee that you receive the most up-to-date data. When included in the request, Square
            returns the specified version or a higher version if one exists. If the specified version is
            higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveBookingCustomAttributeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.custom_attributes.get(
                booking_id="booking_id",
                key="key",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(
            booking_id, key, with_definition=with_definition, version=version, request_options=request_options
        )
        return response.data

    async def upsert(
        self,
        booking_id: str,
        key: str,
        *,
        custom_attribute: CustomAttributeParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpsertBookingCustomAttributeResponse:
        """
        Upserts a bookings custom attribute.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking_id : str
            The ID of the target [booking](entity:Booking).

        key : str
            The key of the custom attribute to create or update. This key must match the `key` of a
            custom attribute definition in the Square seller account. If the requesting application is not
            the definition owner, you must use the qualified key.

        custom_attribute : CustomAttributeParams
            The custom attribute to create or update, with the following fields:

            - `value`. This value must conform to the `schema` specified by the definition.
            For more information, see [Value data types](https://developer.squareup.com/docs/booking-custom-attributes-api/custom-attributes#value-data-types).

            - `version`. To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control for an update operation, include this optional field and specify the current version
            of the custom attribute.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpsertBookingCustomAttributeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.custom_attributes.upsert(
                booking_id="booking_id",
                key="key",
                custom_attribute={},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.upsert(
            booking_id,
            key,
            custom_attribute=custom_attribute,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    async def delete(
        self, booking_id: str, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteBookingCustomAttributeResponse:
        """
        Deletes a bookings custom attribute.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking_id : str
            The ID of the target [booking](entity:Booking).

        key : str
            The key of the custom attribute to delete. This key must match the `key` of a custom
            attribute definition in the Square seller account. If the requesting application is not the
            definition owner, you must use the qualified key.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteBookingCustomAttributeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.custom_attributes.delete(
                booking_id="booking_id",
                key="key",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(booking_id, key, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .fulfillment_recipient import FulfillmentRecipientParams
import typing


class FulfillmentShipmentDetailsParams(typing_extensions.TypedDict):
    """
    Contains the details necessary to fulfill a shipment order.
    """

    recipient: typing_extensions.NotRequired[FulfillmentRecipientParams]
    """
    Information about the person to receive this shipment fulfillment.
    """

    carrier: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The shipping carrier being used to ship this fulfillment (such as UPS, FedEx, or USPS).
    """

    shipping_note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A note with additional information for the shipping carrier.
    """

    shipping_type: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A description of the type of shipping product purchased from the carrier
    (such as First Class, Priority, or Express).
    """

    tracking_number: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The reference number provided by the carrier to track the shipment's progress.
    """

    tracking_url: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A link to the tracking webpage on the carrier's website.
    """

    placed_at: typing_extensions.NotRequired[str]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the shipment was requested. The timestamp must be in RFC 3339 format
    (for example, "2016-09-04T23:59:33.123Z").
    """

    in_progress_at: typing_extensions.NotRequired[str]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when this fulfillment was moved to the `RESERVED` state, which  indicates that preparation
    of this shipment has begun. The timestamp must be in RFC 3339 format (for example, "2016-09-04T23:59:33.123Z").
    """

    packaged_at: typing_extensions.NotRequired[str]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when this fulfillment was moved to the `PREPARED` state, which indicates that the
    fulfillment is packaged. The timestamp must be in RFC 3339 format (for example, "2016-09-04T23:59:33.123Z").
    """

    expected_shipped_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the shipment is expected to be delivered to the shipping carrier.
    The timestamp must be in RFC 3339 format (for example, "2016-09-04T23:59:33.123Z").
    """

    shipped_at: typing_extensions.NotRequired[str]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when this fulfillment was moved to the `COMPLETED` state, which indicates that
    the fulfillment has been given to the shipping carrier. The timestamp must be in RFC 3339 format
    (for example, "2016-09-04T23:59:33.123Z").
    """

    canceled_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating the shipment was canceled.
    The timestamp must be in RFC 3339 format (for example, "2016-09-04T23:59:33.123Z").
    """

    cancel_reason: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A description of why the shipment was canceled.
    """

    failed_at: typing_extensions.NotRequired[str]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the shipment failed to be completed. The timestamp must be in RFC 3339 format
    (for example, "2016-09-04T23:59:33.123Z").
    """

    failure_reason: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A description of why the shipment failed to be completed.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .scheduled_shift import ScheduledShiftParams
import typing
from .error import ErrorParams


class CreateScheduledShiftResponseParams(typing_extensions.TypedDict):
    """
    Represents a [CreateScheduledShift](api-endpoint:Labor-CreateScheduledShift) response.
    Either `scheduled_shift` or `errors` is present in the response.
    """

    scheduled_shift: typing_extensions.NotRequired[ScheduledShiftParams]
    """
    The new scheduled shift. To make the shift public, call 
    [PublishScheduledShift](api-endpoint:Labor-PublishScheduledShift) or
    [BulkPublishScheduledShifts](api-endpoint:Labor-BulkPublishScheduledShifts).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

V1TenderEntryMethod = typing.Union[
    typing.Literal["MANUAL", "SCANNED", "SQUARE_CASH", "SQUARE_WALLET", "SWIPED", "WEB_FORM", "OTHER"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .subscription import Subscription
from .subscription_action import SubscriptionAction
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SwapPlanResponse(UncheckedBaseModel):
    """
    Defines output parameters in a response of the
    [SwapPlan](api-endpoint:Subscriptions-SwapPlan) endpoint.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Errors encountered during the request.
    """

    subscription: typing.Optional[Subscription] = pydantic.Field(default=None)
    """
    The subscription with the updated subscription plan.
    """

    actions: typing.Optional[typing.List[SubscriptionAction]] = pydantic.Field(default=None)
    """
    A list of a `SWAP_PLAN` action created by the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.break_type import BreakTypeParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_break_type_response import CreateBreakTypeResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.get_break_type_response import GetBreakTypeResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...types.update_break_type_response import UpdateBreakTypeResponse
from ...types.delete_break_type_response import DeleteBreakTypeResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawBreakTypesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        break_type: BreakTypeParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateBreakTypeResponse]:
        """
        Creates a new `BreakType`.

        A `BreakType` is a template for creating `Break` objects.
        You must provide the following values in your request to this
        endpoint:

        - `location_id`
        - `break_name`
        - `expected_duration`
        - `is_paid`

        You can only have three `BreakType` instances per location. If you attempt to add a fourth
        `BreakType` for a location, an `INVALID_REQUEST_ERROR` "Exceeded limit of 3 breaks per location."
        is returned.

        Parameters
        ----------
        break_type : BreakTypeParams
            The `BreakType` to be created.

        idempotency_key : typing.Optional[str]
            A unique string value to ensure the idempotency of the operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateBreakTypeResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/labor/break-types",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "break_type": convert_and_respect_annotation_metadata(
                    object_=break_type, annotation=BreakTypeParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateBreakTypeResponse,
                    construct_type(
                        type_=CreateBreakTypeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetBreakTypeResponse]:
        """
        Returns a single `BreakType` specified by `id`.

        Parameters
        ----------
        id : str
            The UUID for the `BreakType` being retrieved.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetBreakTypeResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/labor/break-types/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBreakTypeResponse,
                    construct_type(
                        type_=GetBreakTypeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self, id: str, *, break_type: BreakTypeParams, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[UpdateBreakTypeResponse]:
        """
        Updates an existing `BreakType`.

        Parameters
        ----------
        id : str
             The UUID for the `BreakType` being updated.

        break_type : BreakTypeParams
            The updated `BreakType`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateBreakTypeResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/labor/break-types/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "break_type": convert_and_respect_annotation_metadata(
                    object_=break_type, annotation=BreakTypeParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBreakTypeResponse,
                    construct_type(
                        type_=UpdateBreakTypeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteBreakTypeResponse]:
        """
        Deletes an existing `BreakType`.

        A `BreakType` can be deleted even if it is referenced from a `Shift`.

        Parameters
        ----------
        id : str
            The UUID for the `BreakType` being deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteBreakTypeResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/labor/break-types/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteBreakTypeResponse,
                    construct_type(
                        type_=DeleteBreakTypeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawBreakTypesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        break_type: BreakTypeParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateBreakTypeResponse]:
        """
        Creates a new `BreakType`.

        A `BreakType` is a template for creating `Break` objects.
        You must provide the following values in your request to this
        endpoint:

        - `location_id`
        - `break_name`
        - `expected_duration`
        - `is_paid`

        You can only have three `BreakType` instances per location. If you attempt to add a fourth
        `BreakType` for a location, an `INVALID_REQUEST_ERROR` "Exceeded limit of 3 breaks per location."
        is returned.

        Parameters
        ----------
        break_type : BreakTypeParams
            The `BreakType` to be created.

        idempotency_key : typing.Optional[str]
            A unique string value to ensure the idempotency of the operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateBreakTypeResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/labor/break-types",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "break_type": convert_and_respect_annotation_metadata(
                    object_=break_type, annotation=BreakTypeParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateBreakTypeResponse,
                    construct_type(
                        type_=CreateBreakTypeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetBreakTypeResponse]:
        """
        Returns a single `BreakType` specified by `id`.

        Parameters
        ----------
        id : str
            The UUID for the `BreakType` being retrieved.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetBreakTypeResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/labor/break-types/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBreakTypeResponse,
                    construct_type(
                        type_=GetBreakTypeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self, id: str, *, break_type: BreakTypeParams, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[UpdateBreakTypeResponse]:
        """
        Updates an existing `BreakType`.

        Parameters
        ----------
        id : str
             The UUID for the `BreakType` being updated.

        break_type : BreakTypeParams
            The updated `BreakType`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateBreakTypeResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/labor/break-types/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "break_type": convert_and_respect_annotation_metadata(
                    object_=break_type, annotation=BreakTypeParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBreakTypeResponse,
                    construct_type(
                        type_=UpdateBreakTypeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteBreakTypeResponse]:
        """
        Deletes an existing `BreakType`.

        A `BreakType` can be deleted even if it is referenced from a `Shift`.

        Parameters
        ----------
        id : str
            The UUID for the `BreakType` being deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteBreakTypeResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/labor/break-types/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteBreakTypeResponse,
                    construct_type(
                        type_=DeleteBreakTypeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

V1TenderType = typing.Union[
    typing.Literal[
        "CREDIT_CARD", "CASH", "THIRD_PARTY_CARD", "NO_SALE", "SQUARE_WALLET", "SQUARE_GIFT_CARD", "UNKNOWN", "OTHER"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.list_jobs_response import ListJobsResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..requests.job import JobParams
from ..types.create_job_response import CreateJobResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..types.retrieve_job_response import RetrieveJobResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..types.update_job_response import UpdateJobResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTeamClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_jobs(
        self, *, cursor: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ListJobsResponse]:
        """
        Lists jobs in a seller account. Results are sorted by title in ascending order.

        Parameters
        ----------
        cursor : typing.Optional[str]
            The pagination cursor returned by the previous call to this endpoint. Provide this
            cursor to retrieve the next page of results for your original request. For more information,
            see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListJobsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/team-members/jobs",
            method="GET",
            params={
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListJobsResponse,
                    construct_type(
                        type_=ListJobsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_job(
        self, *, job: JobParams, idempotency_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CreateJobResponse]:
        """
        Creates a job in a seller account. A job defines a title and tip eligibility. Note that
        compensation is defined in a [job assignment](entity:JobAssignment) in a team member's wage setting.

        Parameters
        ----------
        job : JobParams
            The job to create. The `title` field is required and `is_tip_eligible` defaults to true.

        idempotency_key : str
            A unique identifier for the `CreateJob` request. Keys can be any valid string,
            but must be unique for each request. For more information, see
            [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateJobResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/team-members/jobs",
            method="POST",
            json={
                "job": convert_and_respect_annotation_metadata(object_=job, annotation=JobParams, direction="write"),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateJobResponse,
                    construct_type(
                        type_=CreateJobResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve_job(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RetrieveJobResponse]:
        """
        Retrieves a specified job.

        Parameters
        ----------
        job_id : str
            The ID of the job to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RetrieveJobResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/team-members/jobs/{jsonable_encoder(job_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveJobResponse,
                    construct_type(
                        type_=RetrieveJobResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_job(
        self, job_id: str, *, job: JobParams, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[UpdateJobResponse]:
        """
        Updates the title or tip eligibility of a job. Changes to the title propagate to all
        `JobAssignment`, `Shift`, and `TeamMemberWage` objects that reference the job ID. Changes to
        tip eligibility propagate to all `TeamMemberWage` objects that reference the job ID.

        Parameters
        ----------
        job_id : str
            The ID of the job to update.

        job : JobParams
            The job with the updated fields, either `title`, `is_tip_eligible`, or both. Only changed fields need
            to be included in the request. Optionally include `version` to enable optimistic concurrency control.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateJobResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/team-members/jobs/{jsonable_encoder(job_id)}",
            method="PUT",
            json={
                "job": convert_and_respect_annotation_metadata(object_=job, annotation=JobParams, direction="write"),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateJobResponse,
                    construct_type(
                        type_=UpdateJobResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawTeamClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_jobs(
        self, *, cursor: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ListJobsResponse]:
        """
        Lists jobs in a seller account. Results are sorted by title in ascending order.

        Parameters
        ----------
        cursor : typing.Optional[str]
            The pagination cursor returned by the previous call to this endpoint. Provide this
            cursor to retrieve the next page of results for your original request. For more information,
            see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListJobsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/team-members/jobs",
            method="GET",
            params={
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListJobsResponse,
                    construct_type(
                        type_=ListJobsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_job(
        self, *, job: JobParams, idempotency_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CreateJobResponse]:
        """
        Creates a job in a seller account. A job defines a title and tip eligibility. Note that
        compensation is defined in a [job assignment](entity:JobAssignment) in a team member's wage setting.

        Parameters
        ----------
        job : JobParams
            The job to create. The `title` field is required and `is_tip_eligible` defaults to true.

        idempotency_key : str
            A unique identifier for the `CreateJob` request. Keys can be any valid string,
            but must be unique for each request. For more information, see
            [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateJobResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/team-members/jobs",
            method="POST",
            json={
                "job": convert_and_respect_annotation_metadata(object_=job, annotation=JobParams, direction="write"),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateJobResponse,
                    construct_type(
                        type_=CreateJobResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve_job(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RetrieveJobResponse]:
        """
        Retrieves a specified job.

        Parameters
        ----------
        job_id : str
            The ID of the job to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RetrieveJobResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/team-members/jobs/{jsonable_encoder(job_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveJobResponse,
                    construct_type(
                        type_=RetrieveJobResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_job(
        self, job_id: str, *, job: JobParams, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[UpdateJobResponse]:
        """
        Updates the title or tip eligibility of a job. Changes to the title propagate to all
        `JobAssignment`, `Shift`, and `TeamMemberWage` objects that reference the job ID. Changes to
        tip eligibility propagate to all `TeamMemberWage` objects that reference the job ID.

        Parameters
        ----------
        job_id : str
            The ID of the job to update.

        job : JobParams
            The job with the updated fields, either `title`, `is_tip_eligible`, or both. Only changed fields need
            to be included in the request. Optionally include `version` to enable optimistic concurrency control.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateJobResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/team-members/jobs/{jsonable_encoder(job_id)}",
            method="PUT",
            json={
                "job": convert_and_respect_annotation_metadata(object_=job, annotation=JobParams, direction="write"),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateJobResponse,
                    construct_type(
                        type_=UpdateJobResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class LoyaltyProgramExpirationPolicy(UncheckedBaseModel):
    """
    Describes when the loyalty program expires.
    """

    expiration_duration: str = pydantic.Field()
    """
    The number of months before points expire, in `P[n]M` RFC 3339 duration format. For example, a value of `P12M` represents a duration of 12 months. 
    Points are valid through the last day of the month in which they are scheduled to expire. For example, with a  `P12M` duration, points earned on July 6, 2020 expire on August 1, 2021.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class LoyaltyPromotionIncentivePointsAdditionDataParams(typing_extensions.TypedDict):
    """
    Represents the metadata for a `POINTS_ADDITION` type of [loyalty promotion incentive](entity:LoyaltyPromotionIncentive).
    """

    points_addition: int
    """
    The number of additional points to earn each time the promotion is triggered. For example,
    suppose a purchase qualifies for 5 points from the base loyalty program. If the purchase also
    qualifies for a `POINTS_ADDITION` promotion incentive with a `points_addition` of 3, the buyer
    earns a total of 8 points (5 program points + 3 promotion points = 8 points).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .customer_custom_attribute_filter import CustomerCustomAttributeFilter
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CustomerCustomAttributeFilters(UncheckedBaseModel):
    """
    The custom attribute filters in a set of [customer filters](entity:CustomerFilter) used in a search query. Use this filter
    to search based on [custom attributes](entity:CustomAttribute) that are assigned to customer profiles. For more information, see
    [Search by custom attribute](https://developer.squareup.com/docs/customers-api/use-the-api/search-customers#search-by-custom-attribute).
    """

    filters: typing.Optional[typing.List[CustomerCustomAttributeFilter]] = pydantic.Field(default=None)
    """
    The custom attribute filters. Each filter must specify `key` and include the `filter` field with a type-specific filter,
    the `updated_at` field, or both. The provided keys must be unique within the list of custom attribute filters.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CheckoutLocationSettingsPolicyParams(typing_extensions.TypedDict):
    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID to identify the policy when making changes. You must set the UID for policy updates, but it’s optional when setting new policies.
    """

    title: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The title of the policy. This is required when setting the description, though you can update it in a different request.
    """

    description: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The description of the policy.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.team_member_status import TeamMemberStatus
from .team_member_assigned_locations import TeamMemberAssignedLocationsParams
from .wage_setting import WageSettingParams


class TeamMemberParams(typing_extensions.TypedDict):
    """
    A record representing an individual team member for a business.
    """

    id: typing_extensions.NotRequired[str]
    """
    The unique ID for the team member.
    """

    reference_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A second ID used to associate the team member with an entity in another system.
    """

    is_owner: typing_extensions.NotRequired[bool]
    """
    Whether the team member is the owner of the Square account.
    """

    status: typing_extensions.NotRequired[TeamMemberStatus]
    """
    Describes the status of the team member.
    See [TeamMemberStatus](#type-teammemberstatus) for possible values
    """

    given_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The given name (that is, the first name) associated with the team member.
    """

    family_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The family name (that is, the last name) associated with the team member.
    """

    email_address: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The email address associated with the team member. After accepting the invitation
    from Square, only the team member can change this value.
    """

    phone_number: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The team member's phone number, in E.164 format. For example:
    +14155552671 - the country code is 1 for US
    +551155256325 - the country code is 55 for BR
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the team member was created, in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the team member was last updated, in RFC 3339 format.
    """

    assigned_locations: typing_extensions.NotRequired[TeamMemberAssignedLocationsParams]
    """
    Describes the team member's assigned locations.
    """

    wage_setting: typing_extensions.NotRequired[WageSettingParams]
    """
    Information about the team member's overtime exemption status, job assignments, and compensation.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogModifierOverride(UncheckedBaseModel):
    """
    Options to control how to override the default behavior of the specified modifier.
    """

    modifier_id: str = pydantic.Field()
    """
    The ID of the `CatalogModifier` whose default behavior is being overridden.
    """

    on_by_default: typing.Optional[bool] = pydantic.Field(default=None)
    """
    __Deprecated__: Use `on_by_default_override` instead.
    """

    hidden_online_override: typing.Optional[typing.Optional[typing.Any]] = None
    on_by_default_override: typing.Optional[typing.Optional[typing.Any]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .money import MoneyParams


class GiftCardActivityImportParams(typing_extensions.TypedDict):
    """
    Represents details about an `IMPORT` [gift card activity type](entity:GiftCardActivityType).
    This activity type is used when Square imports a third-party gift card, in which case the
    `gan_source` of the gift card is set to `OTHER`.
    """

    amount_money: MoneyParams
    """
    The balance amount on the imported gift card.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .address import AddressParams
from .order import OrderParams
from .additional_recipient import AdditionalRecipientParams


class CheckoutParams(typing_extensions.TypedDict):
    """
    Square Checkout lets merchants accept online payments for supported
    payment types using a checkout workflow hosted on squareup.com.
    """

    id: typing_extensions.NotRequired[str]
    """
    ID generated by Square Checkout when a new checkout is requested.
    """

    checkout_page_url: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The URL that the buyer's browser should be redirected to after the
    checkout is completed.
    """

    ask_for_shipping_address: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    If `true`, Square Checkout will collect shipping information on your
    behalf and store that information with the transaction information in your
    Square Dashboard.
    
    Default: `false`.
    """

    merchant_support_email: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The email address to display on the Square Checkout confirmation page
    and confirmation email that the buyer can use to contact the merchant.
    
    If this value is not set, the confirmation page and email will display the
    primary email address associated with the merchant's Square account.
    
    Default: none; only exists if explicitly set.
    """

    pre_populate_buyer_email: typing_extensions.NotRequired[typing.Optional[str]]
    """
    If provided, the buyer's email is pre-populated on the checkout page
    as an editable text field.
    
    Default: none; only exists if explicitly set.
    """

    pre_populate_shipping_address: typing_extensions.NotRequired[AddressParams]
    """
    If provided, the buyer's shipping info is pre-populated on the
    checkout page as editable text fields.
    
    Default: none; only exists if explicitly set.
    """

    redirect_url: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The URL to redirect to after checkout is completed with `checkoutId`,
    Square's `orderId`, `transactionId`, and `referenceId` appended as URL
    parameters. For example, if the provided redirect_url is
    `http://www.example.com/order-complete`, a successful transaction redirects
    the customer to:
    
    <pre><code>http://www.example.com/order-complete?checkoutId=xxxxxx&amp;orderId=xxxxxx&amp;referenceId=xxxxxx&amp;transactionId=xxxxxx</code></pre>
    
    If you do not provide a redirect URL, Square Checkout will display an order
    confirmation page on your behalf; however Square strongly recommends that
    you provide a redirect URL so you can verify the transaction results and
    finalize the order through your existing/normal confirmation workflow.
    """

    order: typing_extensions.NotRequired[OrderParams]
    """
    Order to be checked out.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The time when the checkout was created, in RFC 3339 format.
    """

    additional_recipients: typing_extensions.NotRequired[typing.Optional[typing.Sequence[AdditionalRecipientParams]]]
    """
    Additional recipients (other than the merchant) receiving a portion of this checkout.
    For example, fees assessed on the purchase by a third party integration.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .loyalty_account_mapping import LoyaltyAccountMappingParams


class SearchLoyaltyAccountsRequestLoyaltyAccountQueryParams(typing_extensions.TypedDict):
    """
    The search criteria for the loyalty accounts.
    """

    mappings: typing_extensions.NotRequired[typing.Optional[typing.Sequence[LoyaltyAccountMappingParams]]]
    """
    The set of mappings to use in the loyalty account search.  
    
    This cannot be combined with `customer_ids`.  
    
    Max: 30 mappings
    """

    customer_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The set of customer IDs to use in the loyalty account search.  
    
    This cannot be combined with `mappings`.  
    
    Max: 30 customer IDs
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .address import Address
from .vendor_contact import VendorContact
from .vendor_status import VendorStatus
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Vendor(UncheckedBaseModel):
    """
    Represents a supplier to a seller.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique Square-generated ID for the [Vendor](entity:Vendor).
    This field is required when attempting to update a [Vendor](entity:Vendor).
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    An RFC 3339-formatted timestamp that indicates when the
    [Vendor](entity:Vendor) was created.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    An RFC 3339-formatted timestamp that indicates when the
    [Vendor](entity:Vendor) was last updated.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the [Vendor](entity:Vendor).
    This field is required when attempting to create or update a [Vendor](entity:Vendor).
    """

    address: typing.Optional[Address] = pydantic.Field(default=None)
    """
    The address of the [Vendor](entity:Vendor).
    """

    contacts: typing.Optional[typing.List[VendorContact]] = pydantic.Field(default=None)
    """
    The contacts of the [Vendor](entity:Vendor).
    """

    account_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The account number of the [Vendor](entity:Vendor).
    """

    note: typing.Optional[str] = pydantic.Field(default=None)
    """
    A note detailing information about the [Vendor](entity:Vendor).
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    The version of the [Vendor](entity:Vendor).
    """

    status: typing.Optional[VendorStatus] = pydantic.Field(default=None)
    """
    The status of the [Vendor](entity:Vendor).
    See [Status](#type-status) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .order import OrderParams


class PayOrderResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of a request to the
    [PayOrder](api-endpoint:Orders-PayOrder) endpoint.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    order: typing_extensions.NotRequired[OrderParams]
    """
    The paid, updated [order](entity:Order).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .scheduled_shift import ScheduledShiftParams
from .error import ErrorParams


class SearchScheduledShiftsResponseParams(typing_extensions.TypedDict):
    """
    Represents a [SearchScheduledShifts](api-endpoint:Labor-SearchScheduledShifts) response.
    Either `scheduled_shifts` or `errors` is present in the response.
    """

    scheduled_shifts: typing_extensions.NotRequired[typing.Sequence[ScheduledShiftParams]]
    """
    A paginated list of scheduled shifts that match the query conditions.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor used to retrieve the next page of results. This field is present
    only if additional results are available.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

CatalogQuickAmountType = typing.Union[typing.Literal["QUICK_AMOUNT_TYPE_MANUAL", "QUICK_AMOUNT_TYPE_AUTO"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class LoyaltyEventOrderFilterParams(typing_extensions.TypedDict):
    """
    Filter events by the order associated with the event.
    """

    order_id: str
    """
    The ID of the [order](entity:Order) associated with the event.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .catalog_item_food_and_beverage_details_dietary_preference import (
    CatalogItemFoodAndBeverageDetailsDietaryPreference,
)
from .catalog_item_food_and_beverage_details_ingredient import CatalogItemFoodAndBeverageDetailsIngredient
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogItemFoodAndBeverageDetails(UncheckedBaseModel):
    """
    The food and beverage-specific details of a `FOOD_AND_BEV` item.
    """

    calorie_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    The calorie count (in the unit of kcal) for the `FOOD_AND_BEV` type of items.
    """

    dietary_preferences: typing.Optional[typing.List[CatalogItemFoodAndBeverageDetailsDietaryPreference]] = (
        pydantic.Field(default=None)
    )
    """
    The dietary preferences for the `FOOD_AND_BEV` item.
    """

    ingredients: typing.Optional[typing.List[CatalogItemFoodAndBeverageDetailsIngredient]] = pydantic.Field(
        default=None
    )
    """
    The ingredients for the `FOOD_AND_BEV` type item.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .timecard_filter import TimecardFilterParams
from .timecard_sort import TimecardSortParams


class TimecardQueryParams(typing_extensions.TypedDict):
    """
    The parameters of a `Timecard` search query, which includes filter and sort options.
    """

    filter: typing_extensions.NotRequired[TimecardFilterParams]
    """
    Query filter options.
    """

    sort: typing_extensions.NotRequired[TimecardSortParams]
    """
    Sort order details.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .break_type import BreakTypeParams
import typing
from .error import ErrorParams


class UpdateBreakTypeResponseParams(typing_extensions.TypedDict):
    """
    A response to a request to update a `BreakType`. The response contains
    the requested `BreakType` objects and might contain a set of `Error` objects if
    the request resulted in errors.
    """

    break_type: typing_extensions.NotRequired[BreakTypeParams]
    """
    The response object.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .money import Money
import pydantic
from .gift_card_activity_adjust_decrement_reason import GiftCardActivityAdjustDecrementReason
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class GiftCardActivityAdjustDecrement(UncheckedBaseModel):
    """
    Represents details about an `ADJUST_DECREMENT` [gift card activity type](entity:GiftCardActivityType).
    """

    amount_money: Money = pydantic.Field()
    """
    The amount deducted from the gift card balance. This value is a positive integer.
    """

    reason: GiftCardActivityAdjustDecrementReason = pydantic.Field()
    """
    The reason the gift card balance was adjusted.
    See [Reason](#type-reason) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .error_category import ErrorCategory
import pydantic
from .error_code import ErrorCode
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Error(UncheckedBaseModel):
    """
    Represents an error encountered during a request to the Connect API.

    See [Handling errors](https://developer.squareup.com/docs/build-basics/handling-errors) for more information.
    """

    category: ErrorCategory = pydantic.Field()
    """
    The high-level category for the error.
    See [ErrorCategory](#type-errorcategory) for possible values
    """

    code: ErrorCode = pydantic.Field()
    """
    The specific code of the error.
    See [ErrorCode](#type-errorcode) for possible values
    """

    detail: typing.Optional[str] = pydantic.Field(default=None)
    """
    A human-readable description of the error for debugging purposes.
    """

    field: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the field provided in the original request (if any) that
    the error pertains to.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .address import AddressParams
from .invoice_recipient_tax_ids import InvoiceRecipientTaxIdsParams


class InvoiceRecipientParams(typing_extensions.TypedDict):
    """
    Represents a snapshot of customer data. This object stores customer data that is displayed on the invoice
    and that Square uses to deliver the invoice.

    When you provide a customer ID for a draft invoice, Square retrieves the associated customer profile and populates
    the remaining `InvoiceRecipient` fields. You cannot update these fields after the invoice is published.
    Square updates the customer ID in response to a merge operation, but does not update other fields.
    """

    customer_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the customer. This is the customer profile ID that 
    you provide when creating a draft invoice.
    """

    given_name: typing_extensions.NotRequired[str]
    """
    The recipient's given (that is, first) name.
    """

    family_name: typing_extensions.NotRequired[str]
    """
    The recipient's family (that is, last) name.
    """

    email_address: typing_extensions.NotRequired[str]
    """
    The recipient's email address.
    """

    address: typing_extensions.NotRequired[AddressParams]
    """
    The recipient's physical address.
    """

    phone_number: typing_extensions.NotRequired[str]
    """
    The recipient's phone number.
    """

    company_name: typing_extensions.NotRequired[str]
    """
    The name of the recipient's company.
    """

    tax_ids: typing_extensions.NotRequired[InvoiceRecipientTaxIdsParams]
    """
    The recipient's tax IDs. The country of the seller account determines whether this field 
    is available for the customer. For more information, see [Invoice recipient tax IDs](https://developer.squareup.com/docs/invoices-api/overview#recipient-tax-ids).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .customer_segment import CustomerSegment
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ListCustomerSegmentsResponse(UncheckedBaseModel):
    """
    Defines the fields that are included in the response body for requests to the `ListCustomerSegments` endpoint.

    Either `errors` or `segments` is present in a given response (never both).
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    segments: typing.Optional[typing.List[CustomerSegment]] = pydantic.Field(default=None)
    """
    The list of customer segments belonging to the associated Square account.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    A pagination cursor to be used in subsequent calls to `ListCustomerSegments`
    to retrieve the next set of query results. The cursor is only present if the request succeeded and
    additional results are available.
    
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .destination_details import DestinationDetails
from .money import Money
from .processing_fee import ProcessingFee
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class PaymentRefund(UncheckedBaseModel):
    """
    Represents a refund of a payment made using Square. Contains information about
    the original payment and the amount of money refunded.
    """

    id: str = pydantic.Field()
    """
    The unique ID for this refund, generated by Square.
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The refund's status:
    - `PENDING` - Awaiting approval.
    - `COMPLETED` - Successfully completed.
    - `REJECTED` - The refund was rejected.
    - `FAILED` - An error occurred.
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The location ID associated with the payment this refund is attached to.
    """

    unlinked: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Flag indicating whether or not the refund is linked to an existing payment in Square.
    """

    destination_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The destination type for this refund.
    
    Current values include `CARD`, `BANK_ACCOUNT`, `WALLET`, `BUY_NOW_PAY_LATER`, `CASH`,
    `EXTERNAL`, and `SQUARE_ACCOUNT`.
    """

    destination_details: typing.Optional[DestinationDetails] = pydantic.Field(default=None)
    """
    Contains information about the refund destination. This field is populated only if
    `destination_id` is defined in the `RefundPayment` request.
    """

    amount_money: Money = pydantic.Field()
    """
    The amount of money refunded. This amount is specified in the smallest denomination
    of the applicable currency (for example, US dollar amounts are specified in cents).
    """

    app_fee_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount of money the application developer contributed to help cover the refunded amount.
    This amount is specified in the smallest denomination of the applicable currency (for example,
    US dollar amounts are specified in cents). For more information, see
    [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).
    """

    processing_fee: typing.Optional[typing.List[ProcessingFee]] = pydantic.Field(default=None)
    """
    Processing fees and fee adjustments assessed by Square for this refund.
    """

    payment_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the payment associated with this refund.
    """

    order_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the order associated with the refund.
    """

    reason: typing.Optional[str] = pydantic.Field(default=None)
    """
    The reason for the refund.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the refund was created, in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the refund was last updated, in RFC 3339 format.
    """

    team_member_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional ID of the team member associated with taking the payment.
    """

    terminal_refund_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional ID for a Terminal refund.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .measurement_unit_custom import MeasurementUnitCustomParams
from ..types.measurement_unit_area import MeasurementUnitArea
from ..types.measurement_unit_length import MeasurementUnitLength
from ..types.measurement_unit_volume import MeasurementUnitVolume
from ..types.measurement_unit_weight import MeasurementUnitWeight
from ..types.measurement_unit_generic import MeasurementUnitGeneric
from ..types.measurement_unit_time import MeasurementUnitTime
from ..types.measurement_unit_unit_type import MeasurementUnitUnitType


class MeasurementUnitParams(typing_extensions.TypedDict):
    """
    Represents a unit of measurement to use with a quantity, such as ounces
    or inches. Exactly one of the following fields are required: `custom_unit`,
    `area_unit`, `length_unit`, `volume_unit`, and `weight_unit`.
    """

    custom_unit: typing_extensions.NotRequired[MeasurementUnitCustomParams]
    """
    A custom unit of measurement defined by the seller using the Point of Sale
    app or ad-hoc as an order line item.
    """

    area_unit: typing_extensions.NotRequired[MeasurementUnitArea]
    """
    Represents a standard area unit.
    See [MeasurementUnitArea](#type-measurementunitarea) for possible values
    """

    length_unit: typing_extensions.NotRequired[MeasurementUnitLength]
    """
    Represents a standard length unit.
    See [MeasurementUnitLength](#type-measurementunitlength) for possible values
    """

    volume_unit: typing_extensions.NotRequired[MeasurementUnitVolume]
    """
    Represents a standard volume unit.
    See [MeasurementUnitVolume](#type-measurementunitvolume) for possible values
    """

    weight_unit: typing_extensions.NotRequired[MeasurementUnitWeight]
    """
    Represents a standard unit of weight or mass.
    See [MeasurementUnitWeight](#type-measurementunitweight) for possible values
    """

    generic_unit: typing_extensions.NotRequired[MeasurementUnitGeneric]
    """
    Reserved for API integrations that lack the ability to specify a real measurement unit
    See [MeasurementUnitGeneric](#type-measurementunitgeneric) for possible values
    """

    time_unit: typing_extensions.NotRequired[MeasurementUnitTime]
    """
    Represents a standard unit of time.
    See [MeasurementUnitTime](#type-measurementunittime) for possible values
    """

    type: typing_extensions.NotRequired[MeasurementUnitUnitType]
    """
    Represents the type of the measurement unit.
    See [MeasurementUnitUnitType](#type-measurementunitunittype) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing

BusinessAppointmentSettingsAlignmentTime = typing.Union[
    typing.Literal["SERVICE_DURATION", "QUARTER_HOURLY", "HALF_HOURLY", "HOURLY"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .time_range import TimeRangeParams
from .scheduled_shift_workday import ScheduledShiftWorkdayParams
from ..types.scheduled_shift_filter_assignment_status import ScheduledShiftFilterAssignmentStatus
from ..types.scheduled_shift_filter_scheduled_shift_status import ScheduledShiftFilterScheduledShiftStatus


class ScheduledShiftFilterParams(typing_extensions.TypedDict):
    """
    Defines filter criteria for a [SearchScheduledShifts](api-endpoint:Labor-SearchScheduledShifts)
    request. Multiple filters in a query are combined as an `AND` operation.
    """

    location_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    Return shifts for the specified locations. When omitted, shifts for all
    locations are returned. If needed, call [ListLocations](api-endpoint:Locations-ListLocations)
    to get location IDs.
    """

    start: typing_extensions.NotRequired[TimeRangeParams]
    """
    Return shifts whose `start_at` time is within the specified
    time range (inclusive).
    """

    end: typing_extensions.NotRequired[TimeRangeParams]
    """
    Return shifts whose `end_at` time is within the specified
    time range (inclusive).
    """

    workday: typing_extensions.NotRequired[ScheduledShiftWorkdayParams]
    """
    Return shifts based on a workday date range.
    """

    team_member_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    Return shifts assigned to specified team members. If needed, call
    [SearchTeamMembers](api-endpoint:Team-SearchTeamMembers) to get team member IDs.
    
    To return only the shifts assigned to the specified team members, include the
    `assignment_status` filter in the query. Otherwise, all unassigned shifts are
    returned along with shifts assigned to the specified team members.
    """

    assignment_status: typing_extensions.NotRequired[ScheduledShiftFilterAssignmentStatus]
    """
    Return shifts based on whether a team member is assigned. A shift is
    assigned if the `team_member_id` field is populated in the `draft_shift_details`
    or `published_shift details` field of the shift.
    
    To return only draft or published shifts, include the `scheduled_shift_statuses`
    filter in the query.
    See [ScheduledShiftFilterAssignmentStatus](#type-scheduledshiftfilterassignmentstatus) for possible values
    """

    scheduled_shift_statuses: typing_extensions.NotRequired[
        typing.Optional[typing.Sequence[ScheduledShiftFilterScheduledShiftStatus]]
    ]
    """
    Return shifts based on the draft or published status of the shift.
    A shift is published if the `published_shift_details` field is present.
    
    Note that shifts with `draft_shift_details.is_deleted` set to `true` are ignored
    with the `DRAFT` filter.
    See [ScheduledShiftFilterScheduledShiftStatus](#type-scheduledshiftfilterscheduledshiftstatus) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .booking_custom_attribute_delete_response import BookingCustomAttributeDeleteResponseParams
from .error import ErrorParams


class BulkDeleteBookingCustomAttributesResponseParams(typing_extensions.TypedDict):
    """
    Represents a [BulkDeleteBookingCustomAttributes](api-endpoint:BookingCustomAttributes-BulkDeleteBookingCustomAttributes) response,
    which contains a map of responses that each corresponds to an individual delete request.
    """

    values: typing_extensions.NotRequired[typing.Dict[str, BookingCustomAttributeDeleteResponseParams]]
    """
    A map of responses that correspond to individual delete requests. Each response has the
    same ID as the corresponding request and contains `booking_id` and  `errors` field.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawLoyaltyClient
from .accounts.client import AccountsClient
from .programs.client import ProgramsClient
from .rewards.client import RewardsClient
from ..requests.loyalty_event_query import LoyaltyEventQueryParams
from ..core.request_options import RequestOptions
from ..types.search_loyalty_events_response import SearchLoyaltyEventsResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawLoyaltyClient
from .accounts.client import AsyncAccountsClient
from .programs.client import AsyncProgramsClient
from .rewards.client import AsyncRewardsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class LoyaltyClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawLoyaltyClient(client_wrapper=client_wrapper)
        self.accounts = AccountsClient(client_wrapper=client_wrapper)

        self.programs = ProgramsClient(client_wrapper=client_wrapper)

        self.rewards = RewardsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawLoyaltyClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawLoyaltyClient
        """
        return self._raw_client

    def search_events(
        self,
        *,
        query: typing.Optional[LoyaltyEventQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchLoyaltyEventsResponse:
        """
        Searches for loyalty events.

        A Square loyalty program maintains a ledger of events that occur during the lifetime of a
        buyer's loyalty account. Each change in the point balance
        (for example, points earned, points redeemed, and points expired) is
        recorded in the ledger. Using this endpoint, you can search the ledger for events.

        Search results are sorted by `created_at` in descending order.

        Parameters
        ----------
        query : typing.Optional[LoyaltyEventQueryParams]
            A set of one or more predefined query filters to apply when
            searching for loyalty events. The endpoint performs a logical AND to
            evaluate multiple filters and performs a logical OR on arrays
            that specifies multiple field values.

        limit : typing.Optional[int]
            The maximum number of results to include in the response.
            The last page might contain fewer events.
            The default is 30 events.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for your original query.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchLoyaltyEventsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.search_events(
            query={
                "filter": {
                    "order_filter": {"order_id": "PyATxhYLfsMqpVkcKJITPydgEYfZY"}
                }
            },
            limit=30,
        )
        """
        response = self._raw_client.search_events(
            query=query, limit=limit, cursor=cursor, request_options=request_options
        )
        return response.data


class AsyncLoyaltyClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawLoyaltyClient(client_wrapper=client_wrapper)
        self.accounts = AsyncAccountsClient(client_wrapper=client_wrapper)

        self.programs = AsyncProgramsClient(client_wrapper=client_wrapper)

        self.rewards = AsyncRewardsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawLoyaltyClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawLoyaltyClient
        """
        return self._raw_client

    async def search_events(
        self,
        *,
        query: typing.Optional[LoyaltyEventQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchLoyaltyEventsResponse:
        """
        Searches for loyalty events.

        A Square loyalty program maintains a ledger of events that occur during the lifetime of a
        buyer's loyalty account. Each change in the point balance
        (for example, points earned, points redeemed, and points expired) is
        recorded in the ledger. Using this endpoint, you can search the ledger for events.

        Search results are sorted by `created_at` in descending order.

        Parameters
        ----------
        query : typing.Optional[LoyaltyEventQueryParams]
            A set of one or more predefined query filters to apply when
            searching for loyalty events. The endpoint performs a logical AND to
            evaluate multiple filters and performs a logical OR on arrays
            that specifies multiple field values.

        limit : typing.Optional[int]
            The maximum number of results to include in the response.
            The last page might contain fewer events.
            The default is 30 events.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for your original query.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchLoyaltyEventsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.search_events(
                query={
                    "filter": {
                        "order_filter": {"order_id": "PyATxhYLfsMqpVkcKJITPydgEYfZY"}
                    }
                },
                limit=30,
            )


        asyncio.run(main())
        """
        response = await self._raw_client.search_events(
            query=query, limit=limit, cursor=cursor, request_options=request_options
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing

DeviceComponentDetailsExternalPower = typing.Union[
    typing.Literal["AVAILABLE_CHARGING", "AVAILABLE_NOT_IN_USE", "UNAVAILABLE", "AVAILABLE_INSUFFICIENT"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .search_events_sort_field import SearchEventsSortField
import pydantic
from .sort_order import SortOrder
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchEventsSort(UncheckedBaseModel):
    """
    Criteria to sort events by.
    """

    field: typing.Optional[SearchEventsSortField] = pydantic.Field(default=None)
    """
    Sort events by event types.
    See [SearchEventsSortField](#type-searcheventssortfield) for possible values
    """

    order: typing.Optional[SortOrder] = pydantic.Field(default=None)
    """
    The order to use for sorting the events.
    See [SortOrder](#type-sortorder) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CatalogQueryItemVariationsForItemOptionValuesParams(typing_extensions.TypedDict):
    """
    The query filter to return the item variations containing the specified item option value IDs.
    """

    item_option_value_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    A set of `CatalogItemOptionValue` IDs to be used to find associated
    `CatalogItemVariation`s. All ItemVariations that contain all of the given
    Item Option Values (in any order) will be returned.
    """



# This file was auto-generated by Fern from our API Definition.

import json
from typing import Any, Dict, List, Optional

from square.types.error import Error

FALLBACK_ERROR = Error(
    category="API_ERROR",
    code="Unknown",
)


class ApiError(Exception):
    """Exception thrown for any non-2XX API responses."""

    status_code: Optional[int]
    body: Any
    errors: List[Error]

    def __init__(self, *, status_code: Optional[int] = None, body: Any = None):
        """
        Initialize an ApiError.

        Args:
            status_code: HTTP status code
            body: Response body
        """
        message = "API Error"
        self.status_code = status_code
        self.body = body
        self.errors = self._parse_errors(body)

        super().__init__(self._build_message(message, status_code, body))

    def _build_message(
        self, message: str, status_code: Optional[int], body: Any
    ) -> str:
        """
        Build a detailed error message.

        Args:
            message: Base error message
            status_code: HTTP status code
            body: Response body

        Returns:
            Formatted error message
        """
        result = f"{message}\nStatus code: {status_code}"

        if body is None or body == "":
            return result

        if isinstance(body, str):
            return f"{result}\nBody: {body}"

        try:
            return f"{result}\nBody: {json.dumps(body, indent=2)}"
        except (TypeError, ValueError):
            return f"{result}\nBody: {str(body)}"

    def _parse_errors(self, body: Any) -> List[Error]:
        """
        Parse errors from the response body.

        Args:
            body: Response body

        Returns:
            List of Error objects
        """

        if body is None:
            return [FALLBACK_ERROR]

        if isinstance(body, str):
            try:
                body = json.loads(body)
            except json.JSONDecodeError:
                return [FALLBACK_ERROR]

        if hasattr(body, "__dict__"):
            try:
                json_str = json.dumps(body, default=lambda o: o.__dict__)
                body = json.loads(json_str)
            except (TypeError, ValueError):
                return [FALLBACK_ERROR]

        if not isinstance(body, dict):
            return [FALLBACK_ERROR]

        if "errors" in body:
            errors = body["errors"]
            if isinstance(errors, list):
                return [self._parse_error(error) for error in errors]

            return [self._parse_error(errors)]

        return [self._parse_error(body)]

    def _parse_error(self, data: Optional[Dict[str, Any]] = None) -> Error:
        """
        Create an Error object from error data.

        Args:
            error_data: Dictionary containing error information

        Returns:
            Error object
        """
        if not data:
            return FALLBACK_ERROR

        return Error(
            category=data.get("category", FALLBACK_ERROR.category),
            code=data.get("code", data.get("type", FALLBACK_ERROR.code)),
            detail=data.get("detail", data.get("message")),
            field=data.get("field"),
        )

    def __str__(self) -> str:
        return f"status_code: {self.status_code}, body: {self.body}"



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .application_details_external_square_product import ApplicationDetailsExternalSquareProduct
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ApplicationDetails(UncheckedBaseModel):
    """
    Details about the application that took the payment.
    """

    square_product: typing.Optional[ApplicationDetailsExternalSquareProduct] = pydantic.Field(default=None)
    """
    The Square product, such as Square Point of Sale (POS), 
    Square Invoices, or Square Virtual Terminal.
    See [ExternalSquareProduct](#type-externalsquareproduct) for possible values
    """

    application_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square ID assigned to the application used to take the payment. 
    Application developers can use this information to identify payments that 
    their application processed. 
    For example, if a developer uses a custom application to process payments, 
    this field contains the application ID from the Developer Dashboard. 
    If a seller uses a [Square App Marketplace](https://developer.squareup.com/docs/app-marketplace) 
    application to process payments, the field contains the corresponding application ID.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.vendor_status import VendorStatus


class SearchVendorsRequestFilterParams(typing_extensions.TypedDict):
    """
    Defines supported query expressions to search for vendors by.
    """

    name: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The names of the [Vendor](entity:Vendor) objects to retrieve.
    """

    status: typing_extensions.NotRequired[typing.Optional[typing.Sequence[VendorStatus]]]
    """
    The statuses of the [Vendor](entity:Vendor) objects to retrieve.
    See [VendorStatus](#type-vendorstatus) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing

MeasurementUnitWeight = typing.Union[
    typing.Literal[
        "IMPERIAL_WEIGHT_OUNCE",
        "IMPERIAL_POUND",
        "IMPERIAL_STONE",
        "METRIC_MILLIGRAM",
        "METRIC_GRAM",
        "METRIC_KILOGRAM",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .loyalty_account_mapping import LoyaltyAccountMappingParams
from .loyalty_account_expiring_point_deadline import LoyaltyAccountExpiringPointDeadlineParams


class LoyaltyAccountParams(typing_extensions.TypedDict):
    """
    Describes a loyalty account in a [loyalty program](entity:LoyaltyProgram). For more information, see
    [Create and Retrieve Loyalty Accounts](https://developer.squareup.com/docs/loyalty-api/loyalty-accounts).
    """

    id: typing_extensions.NotRequired[str]
    """
    The Square-assigned ID of the loyalty account.
    """

    program_id: str
    """
    The Square-assigned ID of the [loyalty program](entity:LoyaltyProgram) to which the account belongs.
    """

    balance: typing_extensions.NotRequired[int]
    """
    The available point balance in the loyalty account. If points are scheduled to expire, they are listed in the `expiring_point_deadlines` field.
    
    Your application should be able to handle loyalty accounts that have a negative point balance (`balance` is less than 0). This might occur if a seller makes a manual adjustment or as a result of a refund or exchange.
    """

    lifetime_points: typing_extensions.NotRequired[int]
    """
    The total points accrued during the lifetime of the account.
    """

    customer_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-assigned ID of the [customer](entity:Customer) that is associated with the account.
    """

    enrolled_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The timestamp when the buyer joined the loyalty program, in RFC 3339 format. This field is used to display the **Enrolled On** or **Member Since** date in first-party Square products.
    
    If this field is not set in a `CreateLoyaltyAccount` request, Square populates it after the buyer's first action on their account 
    (when `AccumulateLoyaltyPoints` or `CreateLoyaltyReward` is called). In first-party flows, Square populates the field when the buyer agrees to the terms of service in Square Point of Sale. 
    
    This field is typically specified in a `CreateLoyaltyAccount` request when creating a loyalty account for a buyer who already interacted with their account. 
    For example, you would set this field when migrating accounts from an external system. The timestamp in the request can represent a current or previous date and time, but it cannot be set for the future.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the loyalty account was created, in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the loyalty account was last updated, in RFC 3339 format.
    """

    mapping: typing_extensions.NotRequired[LoyaltyAccountMappingParams]
    """
    The mapping that associates the loyalty account with a buyer. Currently,
    a loyalty account can only be mapped to a buyer by phone number.
    
    To create a loyalty account, you must specify the `mapping` field, with the buyer's phone number
    in the `phone_number` field.
    """

    expiring_point_deadlines: typing_extensions.NotRequired[
        typing.Optional[typing.Sequence[LoyaltyAccountExpiringPointDeadlineParams]]
    ]
    """
    The schedule for when points expire in the loyalty account balance. This field is present only if the account has points that are scheduled to expire. 
    
    The total number of points in this field equals the number of points in the `balance` field.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .appointment_segment import AppointmentSegmentParams


class AvailabilityParams(typing_extensions.TypedDict):
    """
    Defines an appointment slot that encapsulates the appointment segments, location and starting time available for booking.
    """

    start_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The RFC 3339 timestamp specifying the beginning time of the slot available for booking.
    """

    location_id: typing_extensions.NotRequired[str]
    """
    The ID of the location available for booking.
    """

    appointment_segments: typing_extensions.NotRequired[typing.Optional[typing.Sequence[AppointmentSegmentParams]]]
    """
    The list of appointment segments available for booking
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .custom_attribute_definition import CustomAttributeDefinitionParams
from .error import ErrorParams


class ListCustomerCustomAttributeDefinitionsResponseParams(typing_extensions.TypedDict):
    """
    Represents a [ListCustomerCustomAttributeDefinitions](api-endpoint:CustomerCustomAttributes-ListCustomerCustomAttributeDefinitions) response.
    Either `custom_attribute_definitions`, an empty object, or `errors` is present in the response.
    If additional results are available, the `cursor` field is also present along with `custom_attribute_definitions`.
    """

    custom_attribute_definitions: typing_extensions.NotRequired[typing.Sequence[CustomAttributeDefinitionParams]]
    """
    The retrieved custom attribute definitions. If no custom attribute definitions are found,
    Square returns an empty object (`{}`).
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The cursor to provide in your next call to this endpoint to retrieve the next page of
    results for your original request. This field is present only if the request succeeded and
    additional results are available. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .gift_card import GiftCardParams


class ListGiftCardsResponseParams(typing_extensions.TypedDict):
    """
    A response that contains a list of `GiftCard` objects. If the request resulted in errors,
    the response contains a set of `Error` objects.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    gift_cards: typing_extensions.NotRequired[typing.Sequence[GiftCardParams]]
    """
    The requested gift cards or an empty object if none are found.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    When a response is truncated, it includes a cursor that you can use in a
    subsequent request to retrieve the next set of gift cards. If a cursor is not present, this is
    the final response.
    For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.catalog_quick_amount_type import CatalogQuickAmountType
from .money import MoneyParams
import typing_extensions
import typing


class CatalogQuickAmountParams(typing_extensions.TypedDict):
    """
    Represents a Quick Amount in the Catalog.
    """

    type: CatalogQuickAmountType
    """
    Represents the type of the Quick Amount.
    See [CatalogQuickAmountType](#type-catalogquickamounttype) for possible values
    """

    amount: MoneyParams
    """
    Represents the actual amount of the Quick Amount with Money type.
    """

    score: typing_extensions.NotRequired[typing.Optional[int]]
    """
    Describes the ranking of the Quick Amount provided by machine learning model, in the range [0, 100].
    MANUAL type amount will always have score = 100.
    """

    ordinal: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The order in which this Quick Amount should be displayed.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .bulk_upsert_merchant_custom_attributes_response_merchant_custom_attribute_upsert_response import (
    BulkUpsertMerchantCustomAttributesResponseMerchantCustomAttributeUpsertResponse,
)
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BulkUpsertMerchantCustomAttributesResponse(UncheckedBaseModel):
    """
    Represents a [BulkUpsertMerchantCustomAttributes](api-endpoint:MerchantCustomAttributes-BulkUpsertMerchantCustomAttributes) response,
    which contains a map of responses that each corresponds to an individual upsert request.
    """

    values: typing.Optional[
        typing.Dict[str, BulkUpsertMerchantCustomAttributesResponseMerchantCustomAttributeUpsertResponse]
    ] = pydantic.Field(default=None)
    """
    A map of responses that correspond to individual upsert requests. Each response has the
    same ID as the corresponding request and contains either a `merchant_id` and `custom_attribute` or an `errors` field.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class LoyaltyAccountMappingParams(typing_extensions.TypedDict):
    """
    Represents the mapping that associates a loyalty account with a buyer.

    Currently, a loyalty account can only be mapped to a buyer by phone number. For more information, see
    [Loyalty Overview](https://developer.squareup.com/docs/loyalty/overview).
    """

    id: typing_extensions.NotRequired[str]
    """
    The Square-assigned ID of the mapping.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the mapping was created, in RFC 3339 format.
    """

    phone_number: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The phone number of the buyer, in E.164 format. For example, "+14155551111".
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .vendor_status import VendorStatus
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchVendorsRequestFilter(UncheckedBaseModel):
    """
    Defines supported query expressions to search for vendors by.
    """

    name: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The names of the [Vendor](entity:Vendor) objects to retrieve.
    """

    status: typing.Optional[typing.List[VendorStatus]] = pydantic.Field(default=None)
    """
    The statuses of the [Vendor](entity:Vendor) objects to retrieve.
    See [VendorStatus](#type-vendorstatus) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class QuickPay(UncheckedBaseModel):
    """
    Describes an ad hoc item and price to generate a quick pay checkout link.
    For more information,
    see [Quick Pay Checkout](https://developer.squareup.com/docs/checkout-api/quick-pay-checkout).
    """

    name: str = pydantic.Field()
    """
    The ad hoc item name. In the resulting `Order`, this name appears as the line item name.
    """

    price_money: Money = pydantic.Field()
    """
    The price of the item.
    """

    location_id: str = pydantic.Field()
    """
    The ID of the business location the checkout is associated with.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SignatureImage(UncheckedBaseModel):
    image_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The mime/type of the image data.
    Use `image/png;base64` for png.
    """

    data: typing.Optional[str] = pydantic.Field(default=None)
    """
    The base64 representation of the image.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class SearchSubscriptionsFilterParams(typing_extensions.TypedDict):
    """
    Represents a set of query expressions (filters) to narrow the scope of targeted subscriptions returned by
    the [SearchSubscriptions](api-endpoint:Subscriptions-SearchSubscriptions) endpoint.
    """

    customer_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    A filter to select subscriptions based on the subscribing customer IDs.
    """

    location_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    A filter to select subscriptions based on the location.
    """

    source_names: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    A filter to select subscriptions based on the source application.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing_extensions
import typing
from ..core.serialization import FieldMetadata
import pydantic
from .country import Country
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Address(UncheckedBaseModel):
    """
    Represents a postal address in a country.
    For more information, see [Working with Addresses](https://developer.squareup.com/docs/build-basics/working-with-addresses).
    """

    address_line1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="address_line_1")] = (
        pydantic.Field(default=None)
    )
    """
    The first line of the address.
    
    Fields that start with `address_line` provide the address's most specific
    details, like street number, street name, and building name. They do *not*
    provide less specific details like city, state/province, or country (these
    details are provided in other fields).
    """

    address_line2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="address_line_2")] = (
        pydantic.Field(default=None)
    )
    """
    The second line of the address, if any.
    """

    address_line3: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="address_line_3")] = (
        pydantic.Field(default=None)
    )
    """
    The third line of the address, if any.
    """

    locality: typing.Optional[str] = pydantic.Field(default=None)
    """
    The city or town of the address. For a full list of field meanings by country, see [Working with Addresses](https://developer.squareup.com/docs/build-basics/working-with-addresses).
    """

    sublocality: typing.Optional[str] = pydantic.Field(default=None)
    """
    A civil region within the address's `locality`, if any.
    """

    sublocality2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="sublocality_2")] = (
        pydantic.Field(default=None)
    )
    """
    A civil region within the address's `sublocality`, if any.
    """

    sublocality3: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="sublocality_3")] = (
        pydantic.Field(default=None)
    )
    """
    A civil region within the address's `sublocality_2`, if any.
    """

    administrative_district_level1: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="administrative_district_level_1")
    ] = pydantic.Field(default=None)
    """
    A civil entity within the address's country. In the US, this
    is the state. For a full list of field meanings by country, see [Working with Addresses](https://developer.squareup.com/docs/build-basics/working-with-addresses).
    """

    administrative_district_level2: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="administrative_district_level_2")
    ] = pydantic.Field(default=None)
    """
    A civil entity within the address's `administrative_district_level_1`.
    In the US, this is the county.
    """

    administrative_district_level3: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="administrative_district_level_3")
    ] = pydantic.Field(default=None)
    """
    A civil entity within the address's `administrative_district_level_2`,
    if any.
    """

    postal_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The address's postal code. For a full list of field meanings by country, see [Working with Addresses](https://developer.squareup.com/docs/build-basics/working-with-addresses).
    """

    country: typing.Optional[Country] = pydantic.Field(default=None)
    """
    The address's country, in the two-letter format of ISO 3166. For example, `US` or `FR`.
    See [Country](#type-country) for possible values
    """

    first_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional first name when it's representing recipient.
    """

    last_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional last name when it's representing recipient.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams
from ..types.catalog_pricing_type import CatalogPricingType
from ..types.inventory_alert_type import InventoryAlertType


class ItemVariationLocationOverridesParams(typing_extensions.TypedDict):
    """
    Price and inventory alerting overrides for a `CatalogItemVariation` at a specific `Location`.
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the `Location`. This can include locations that are deactivated.
    """

    price_money: typing_extensions.NotRequired[MoneyParams]
    """
    The price of the `CatalogItemVariation` at the given `Location`, or blank for variable pricing.
    """

    pricing_type: typing_extensions.NotRequired[CatalogPricingType]
    """
    The pricing type (fixed or variable) for the `CatalogItemVariation` at the given `Location`.
    See [CatalogPricingType](#type-catalogpricingtype) for possible values
    """

    track_inventory: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    If `true`, inventory tracking is active for the `CatalogItemVariation` at this `Location`.
    """

    inventory_alert_type: typing_extensions.NotRequired[InventoryAlertType]
    """
    Indicates whether the `CatalogItemVariation` displays an alert when its inventory
    quantity is less than or equal to its `inventory_alert_threshold`.
    See [InventoryAlertType](#type-inventoryalerttype) for possible values
    """

    inventory_alert_threshold: typing_extensions.NotRequired[typing.Optional[int]]
    """
    If the inventory quantity for the variation is less than or equal to this value and `inventory_alert_type`
    is `LOW_QUANTITY`, the variation displays an alert in the merchant dashboard.
    
    This value is always an integer.
    """

    sold_out: typing_extensions.NotRequired[bool]
    """
    Indicates whether the overridden item variation is sold out at the specified location.
    
    When inventory tracking is enabled on the item variation either globally or at the specified location,
    the item variation is automatically marked as sold out when its inventory count reaches zero. The seller
    can manually set the item variation as sold out even when the inventory count is greater than zero.
    Attempts by an application to set this attribute are ignored. Regardless how the sold-out status is set,
    applications should treat its inventory count as zero when this attribute value is `true`.
    """

    sold_out_valid_until: typing_extensions.NotRequired[str]
    """
    The seller-assigned timestamp, of the RFC 3339 format, to indicate when this sold-out variation
    becomes available again at the specified location. Attempts by an application to set this attribute are ignored.
    When the current time is later than this attribute value, the affected item variation is no longer sold out.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .timecard_filter_status import TimecardFilterStatus
from .time_range import TimeRange
from .timecard_workday import TimecardWorkday
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TimecardFilter(UncheckedBaseModel):
    """
    Defines a filter used in a search for `Timecard` records. `AND` logic is
    used by Square's servers to apply each filter property specified.
    """

    location_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Fetch timecards for the specified location.
    """

    status: typing.Optional[TimecardFilterStatus] = pydantic.Field(default=None)
    """
    Fetch a `Timecard` instance by `Timecard.status`.
    See [TimecardFilterStatus](#type-timecardfilterstatus) for possible values
    """

    start: typing.Optional[TimeRange] = pydantic.Field(default=None)
    """
    Fetch `Timecard` instances that start in the time range - Inclusive.
    """

    end: typing.Optional[TimeRange] = pydantic.Field(default=None)
    """
    Fetch the `Timecard` instances that end in the time range - Inclusive.
    """

    workday: typing.Optional[TimecardWorkday] = pydantic.Field(default=None)
    """
    Fetch the `Timecard` instances based on the workday date range.
    """

    team_member_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Fetch timecards for the specified team members.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .order_return_line_item import OrderReturnLineItemParams
from .order_return_service_charge import OrderReturnServiceChargeParams
from .order_return_tax import OrderReturnTaxParams
from .order_return_discount import OrderReturnDiscountParams
from .order_return_tip import OrderReturnTipParams
from .order_rounding_adjustment import OrderRoundingAdjustmentParams
from .order_money_amounts import OrderMoneyAmountsParams


class OrderReturnParams(typing_extensions.TypedDict):
    """
    The set of line items, service charges, taxes, discounts, tips, and other items being returned in an order.
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID that identifies the return only within this order.
    """

    source_order_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    An order that contains the original sale of these return line items. This is unset
    for unlinked returns.
    """

    return_line_items: typing_extensions.NotRequired[typing.Optional[typing.Sequence[OrderReturnLineItemParams]]]
    """
    A collection of line items that are being returned.
    """

    return_service_charges: typing_extensions.NotRequired[
        typing.Optional[typing.Sequence[OrderReturnServiceChargeParams]]
    ]
    """
    A collection of service charges that are being returned.
    """

    return_taxes: typing_extensions.NotRequired[typing.Sequence[OrderReturnTaxParams]]
    """
    A collection of references to taxes being returned for an order, including the total
    applied tax amount to be returned. The taxes must reference a top-level tax ID from the source
    order.
    """

    return_discounts: typing_extensions.NotRequired[typing.Sequence[OrderReturnDiscountParams]]
    """
    A collection of references to discounts being returned for an order, including the total
    applied discount amount to be returned. The discounts must reference a top-level discount ID
    from the source order.
    """

    return_tips: typing_extensions.NotRequired[typing.Optional[typing.Sequence[OrderReturnTipParams]]]
    """
    A collection of references to tips being returned for an order.
    """

    rounding_adjustment: typing_extensions.NotRequired[OrderRoundingAdjustmentParams]
    """
    A positive or negative rounding adjustment to the total value being returned. Adjustments are commonly
    used to apply cash rounding when the minimum unit of the account is smaller than the lowest
    physical denomination of the currency.
    """

    return_amounts: typing_extensions.NotRequired[OrderMoneyAmountsParams]
    """
    An aggregate monetary value being returned by this return entry.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SubscriptionTestResult(UncheckedBaseModel):
    """
    Represents the details of a webhook subscription, including notification URL,
    event types, and signature key.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A Square-generated unique ID for the subscription test result.
    """

    status_code: typing.Optional[int] = pydantic.Field(default=None)
    """
    The status code returned by the subscription notification URL.
    """

    payload: typing.Optional[str] = pydantic.Field(default=None)
    """
    An object containing the payload of the test event. For example, a `payment.created` event.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the subscription was created, in RFC 3339 format. 
    For example, "2016-09-04T23:59:33.123Z".
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the subscription was updated, in RFC 3339 format. For example, "2016-09-04T23:59:33.123Z".
    Because a subscription test result is unique, this field is the same as the `created_at` field.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.product_type import ProductType
from ..types.device_code_status import DeviceCodeStatus


class DeviceCodeParams(typing_extensions.TypedDict):
    id: typing_extensions.NotRequired[str]
    """
    The unique id for this device code.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    An optional user-defined name for the device code.
    """

    code: typing_extensions.NotRequired[str]
    """
    The unique code that can be used to login.
    """

    device_id: typing_extensions.NotRequired[str]
    """
    The unique id of the device that used this code. Populated when the device is paired up.
    """

    product_type: ProductType
    """
    The targeting product type of the device code.
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The location assigned to this code.
    """

    status: typing_extensions.NotRequired[DeviceCodeStatus]
    """
    The pairing status of the device code.
    See [DeviceCodeStatus](#type-devicecodestatus) for possible values
    """

    pair_by: typing_extensions.NotRequired[str]
    """
    When this DeviceCode will expire and no longer login. Timestamp in RFC 3339 format.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    When this DeviceCode was created. Timestamp in RFC 3339 format.
    """

    status_changed_at: typing_extensions.NotRequired[str]
    """
    When this DeviceCode's status was last changed. Timestamp in RFC 3339 format.
    """

    paired_at: typing_extensions.NotRequired[str]
    """
    When this DeviceCode was paired. Timestamp in RFC 3339 format.
    """
